/* align-test.c --- test of alignment
 * use with GCC on x86 with RDTSC instruction (Pentium+,K6,...) 
 * Author: P. Peringer, 1999, Public domain
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * rdtsc() --- reading Pentium+, K6, ... 64 bit timer (GNU C/C++ only)
 */
#ifndef __GNUC__
#error "use GNU C, please"
#endif

__inline long long rdtsc()
{
    register long long edxeax;
    __asm __volatile__ ( "rdtsc" : "=A" (edxeax) );
    return edxeax;
}

#define MAX      100        /* array length (should be small) */
#define OFFSET1    1        /* 0..sizeof(long) */

long long test(long *t, long *x) 
{
  long long start;
  long long end;
  long long dt=1000000;
  int i,n;
  for(n=0;n<10;n++) {
      start = rdtsc(); 
      for(i=0; i<MAX; i++ ) { x[i] = t[i]; } 
      end = rdtsc();
      if(end-start < dt) dt = end-start;  /* minimum */
  }
  return dt;
}

/* main test */
int main(void) {
  long *t = (long *)malloc(MAX*sizeof(long)+OFFSET1);
  long *x = (long *)malloc(MAX*sizeof(long)+OFFSET1);
  printf("Timing of (un)aligned data access\n");
  if( x==NULL || t==NULL ) {
	  printf("not enough memory\n");
	  return 1;
  }
  /* fill caches */
  test(t,x);
  /* aligned access */
  printf("time: %Ld clocks aligned \n", test(t,x));
  /* unaligned access */
  t = (long*)(((char*)t)+OFFSET1); /* offset 1B */
  x = (long*)(((char*)x)+OFFSET1); /* offset 1B */
  printf("time: %Ld clocks unaligned \n", test(t,x));
  return 0;
}

