/* getline.c
 *
 * implementace funkce:
 *   int getline(char *s, int n, FILE *f);
 *
 * pro test prelozte: gcc -DTEST getline.c  
 *
 */

#include <stdio.h>
#include <assert.h>
/* #include "getline.h"  --- doplnit rozhran modulu */

/* 
 * getline(s,n,f) --- funkce pro cteni radku ze souboru
 *
 * parametry:
 *    s - pole pro ulozeni prectenych znaku
 *    n - delka pole znaku (musi byt >1)
 *        pokud je radek delsi, precte pouze n-1 znaku a zbytek preskoci
 *    f - soubor otevreny v textovem rezimu
 *
 * vraci: 
 *    delku retezce s nebo EOF
 * 
 */
int getline(char *s, int n, FILE * f)
{
    int i = 0;
    int c;
    n--;

    assert(n > 0);		/* kontrola */

    while ((c = getc(f)) != EOF) {
	if (c == '\n')		/* '\n' se do vysledku neulozi */
	    break;
	if (i < n)
	    s[i++] = c;
    }

    if (i == 0 && c == EOF)	/* konec souboru */
	return EOF;

    s[i] = '\0';

    return i;			/* vraci delku retezce s */
}

/**************************************************************************/
#ifdef TEST			/* test funkce */

#define N 80

int main(int argc, char *argv[])
{
    FILE *f = stdin;
    char s[N];			/* pole pro znaky */
    int len;
    long ln = 0;

    if (argc > 2) {
	fprintf(stderr, "Usage: %s [file]\n\n", argv[0]);
	exit(1);
    }

    if (argc == 2) {
	f = fopen(argv[1], "r");
	if (f == NULL) {
	    fprintf(stderr, "Can't open file: %s\n", argv[1]);
	    exit(2);
	}
    }

    while ((len = getline(s, sizeof(s), f)) != EOF) {
	/* zpracovani radku */
	ln++;
	printf("%5ld %2d |%s|\n", ln, len, s);
    }

    return 0;
}

#endif				/* TEST */
