// smap.cc -- ablona Map = asociativn pole
//
// podle knihy
//   Stroustrup: The C++ Programming Language, 2nd edition. (odst.8.8)
//
// program na cin oekv:
//   slovo slo
//   ......
//   <EOF>
//

#include <string>
#include <iostream>
#include <cstdlib> // malloc(), free(), exit()
using namespace std;

void error(char *s) {  // hlen chyby a konec
  cerr << endl << "ERROR: " << s << endl;
  exit(1);
}

// nepln deklarace ablon
template<class K, class V> class Map;
template<class K, class V> class Mapiter;

// ablona Link -- prvek seznamu = dvojice (kl,hodnota)
template<class K, class V>
class Link {
  friend class Map<K,V>;
  friend class Mapiter<K,V>;
  const K key;                  // konstantn kl
  V value;                      // hodnota
  Link *pre;                    // vazba na pedchoz prvek
  Link *suc;                    // vazba na nsledujc prvek
  Link(const K& k, const V& v) : key(k), value(v) {}
  ~Link() { delete suc; }       // rume rekurzivn... ????
};

// ablona Map -- asociativn pole implementovan seznamem
template<class K, class V>
class Map {
  friend class Mapiter<K,V>;    // itertor
  Link<K,V> *head;              // seznam
  Link<K,V> *current;           // aktuln prvek
  K def_key;                    // implicitn kl
  V def_val;                    // implicitn hodnota
  void init() { head=0; current=0; }
  Map(const Map&);              // zakzan operace
  Map& operator= (const Map&);
 public:
  Map() { init(); }             // inicializace
  Map(const K& k, const V& d) : def_key(k), def_val(d) { init(); }
  ~Map() { delete head; }       // rume rekurzivn...
  V& operator[] (const K&);     // zskn hodnoty podle kle
  Mapiter<K,V> first() { return Mapiter<K,V>(this,head); }
};

// ablona Mapiter -- itertor pro prochzen asociativnm polem
template<class K, class V>
class Mapiter {
  friend class Map<K,V>;
  Map<K,V> *m;                  // pole
  Link<K,V> *p;                 // aktuln poloka
 public:
  Mapiter() { m = 0; p = 0; }
  Mapiter(Map<K,V> *mm)                { m = mm; p = m->head; }
  Mapiter(Map<K,V> *mm, Link<K,V> *pp) { m = mm; p = pp; }
  Mapiter(Map<K,V> &mm)                { m = &mm; p = m->head; }
  Mapiter(Map<K,V> &mm, Link<K,V> &pp) { m = &mm; p = &pp; }
  operator void*() { return p; }// pro testovn konce prchodu
  const K& key();               // kl
  V& value();                   // hodnota
  Mapiter& operator--();        // posuny...
  void operator--(int);
  Mapiter& operator++();
  void operator++(int);
};

// definice metod ablon...
template<class K, class V>
V& Map<K,V>::operator[](const K& k)
{
  if(head==0)                   // przdn seznam
  {                             // vlo s implicitn hodnotou
    current = head = new Link<K,V>(k,def_val);
    current->pre = current->suc = 0;
    return current->value;
  }
  Link<K,V> *p = head;          // neprzdn seznam
  for(;;) {
    if(p->key == k) {           // nalezeno
      current = p;              // nastav aktuln prvek
      return current->value;    // vrac hodnotu
    }
    if(k < p->key) {            // vlo ped p
      current = new Link<K,V>(k,def_val);
      current->pre = p->pre;
      current->suc = p;
      if(p==head)               // je prvn?
        head = current;
      else
        p->pre->suc = current;
      p->pre = current;
      return current->value;
    }
    Link<K,V> *s = p->suc;
    if(s == 0) {                // vlo na konec
      current = new Link<K,V>(k,def_val);
      current->pre = p;
      current->suc = 0;
      p->suc = current;
      return current->value;
    }
    p = s;                      // posun na dal
  } // for(;;)
}


template<class K, class V>
const K& Mapiter<K,V>::key()    // aktuln kl
{
  if(p) return p->key;
  else  return m->def_key;
}

template<class K, class V>
V& Mapiter<K,V>::value()        // aktuln hodnota
{
  if(p) return p->value;
  else  return m->def_val;
}

template<class K, class V>
Mapiter<K,V>& Mapiter<K,V>::operator--()
{
  if(p) p = p->pre;
  return *this;
}

template<class K, class V>
void Mapiter<K,V>::operator--(int) // postfix
{
  if(p) p = p->pre;
}

template<class K, class V>
Mapiter<K,V>& Mapiter<K,V>::operator++()
{
  if(p) p = p->suc;
  return *this;
}

template<class K, class V>
void Mapiter<K,V>::operator++(int) // postfix
{
  if(p) p = p->suc;
}

// tda String -- jednoduch etzce
class String {
  enum { MAXSIZE = 80 };
  char s[MAXSIZE+1];
 public:
  String () {                   // implicitn konstruktor
    strcpy(s,"");
  }
  String (const String &str) {  // copy konstruktor
    strcpy(s,str.s);
  }
  String (char *str) {          // konstruktor
    if(strlen(str)>MAXSIZE) error("string too long");
    strcpy(s,str);
  }
  String &operator = (String &str) {    // opertor piazen
    strcpy(s,str.s);
    return *this;
  }
  int operator == (const String& str) const { return strcmp(s,str.s)==0; }
  int operator < (const String& str) const { return strcmp(s,str.s)<0; }
  // operace vstupu
  friend ostream& operator << (ostream &x, const String &s);
  friend istream& operator >> (istream &x, String &s);
};

// vstup-vstupn operace pro tdu String
ostream& operator << (ostream &x, const String &s)
  { return x << s.s; }

istream& operator >> (istream &x, String &s)
  { return x >> s.s; }

// ablona funkce readlines -- ten dk do asoc. pole
template<class K, class V>
void readlines(Map<K,V>& key)
{
  K word;
  while(cin >> word) {
    V val = 0;
    if(cin >> val)
      key[word] += val;
    else
      return;
  }
}

////////////////////////////////////////////////////////////////////////////
// funkce main -- test pouit ablon
//
int main() {
  Map<String,int> tbl("nil",0);
  readlines(tbl);                       // ten ze vstupu

  int total = 0;
  for(Mapiter<String,int> p(tbl); p; ++p) {
    int val = p.value();
    total += val;
    cout << p.key() << '\t' << val << '\n' ;
  }
  cout << "----------------" << endl;
  cout << "total: " << total << endl;
  return 0;
}

